///----------------------------------------------------------
/// Projekt: DrumPlayer, 2012
/// Popis: Multimedialni aplikace pro Kinect. Hrani na bici.
/// Autori: Vendula Poncova, xponco00@stud.fit.vutbr.cz
///         Marek Salat, xsalat00@stud.fit.vutbr.cz
///         Roman Luks, xluksr00@stud.fit.vutbr.cz
///         Pavel Kefurt, xkefur00@stud.fit.vutbr.cz
///
/// Soubor: DrumStick.xaml.cs
/// Popis: Reprezentace hracich palicek. Umoznuje kontrolu gesta pro bubnovani na bici soustavu.
///-----------------------------------------------------------------------------------------------

using System;
using System.Diagnostics;
using Microsoft.Kinect;
using System.Media;
using System.Windows.Media;
using myDrum;
using myDrumSet;

namespace myDrumStick
{
    public partial class DrumStick
    {   
        // bici soustava
        protected DrumSet drumset;
        
        // posledni buben, na ktery se hralo
        protected Drum lastDrum;
        
        // senzor
        protected KinectSensor sensor;
     
        // pomocne promenne pro kontrolu gesta
        protected Joint lastPoint;
        protected float lastY;
        protected int direction;
        protected float eps = .001f;   
        
        // pocet uderu palickou     
        protected int hits = 0;

        // kontruktor
        public DrumStick(KinectSensor sensor, DrumSet drumset)       {
            this.drumset = drumset;
            this.sensor = sensor;
            lastY = .0f;
            direction = 0;
        }
        
        // vrati pocet uderu palickou
        public int getHits(){
            return this.hits;
        }

        // kontrola gesta bubnovani
        public void checkGest(Joint j)        {
            float currY = j.Position.Y;

            // doslo k pohybu
            if (Math.Abs(currY - this.lastY) > this.eps)
            {
                // byla zmena smeru z nahoru na dolu
                // nastaveni smeru
                if (this.lastY > currY)
                {
                    this.direction = -1;
                }
                // zmena smeru z dolu na nahoru
                // nastaveni smeru a kontrola kolize
                else
                {
                    if (direction <= 0){
                        DepthImagePoint depthPoint = sensor.MapSkeletonPointToDepth(lastPoint.Position, DepthImageFormat.Resolution640x480Fps30);
                        bool collDetect = false;
                        
                        // detekce kolize pro kazdy buben hracii soustavy
                        foreach (Drum drum in this.drumset.getDrums()) {
                        
                            // pokud doslo ke kolizi
                            if (drum.collision(depthPoint.X, 480 - depthPoint.Y)) {
                                collDetect = true;
                                
                                // pokud lze prehrat zvuk
                                if (lastDrum != drum){
                                    lastDrum = drum;
                                    
                                    // prehrani zvuku
                                    drum.play();
                                    this.hits++;
                                    //Debug.WriteLine("Hits: " + this.hits);
                                }
                                // netreba dal pokracovat
                                break;
                            }
                        }
                        // nebyla li detekovana kolize tak nemam posledni buben 
                        if (!collDetect)
                            lastDrum = null;
                    }
                    this.direction = 1;
                }
            }
            // zadna zmena pohybu
            else
            {
                //Debug.WriteLine("Stoji.");
                this.direction = 0;
            }
            
            this.lastY = currY;
            this.lastPoint = j;
        }
    }
}