///----------------------------------------------------------
/// Projekt: DrumPlayer, 2012
/// Popis: Multimedialni aplikace pro Kinect. Hrani na bici.
/// Autori: Vendula Poncova, xponco00@stud.fit.vutbr.cz
///         Marek Salat, xsalat00@stud.fit.vutbr.cz
///         Roman Luks, xluksr00@stud.fit.vutbr.cz
///         Pavel Kefurt, xkefur00@stud.fit.vutbr.cz
///
/// Soubor: Window1.xaml.cs
/// Popis: Trida Window1 zpracovava akce uzivatele v aplikaci DrumPlayer.
/// Inicializuje okno aplikace, prostredi Kinectu, kostru uživatele a bici soustavu
/// a definuje rozpoznavani gest a reakce na ne. 
/// Rozhrani aplikace je definovane v souboru Window1.xaml.
///-----------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Documents;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Media;
using System.Diagnostics;
using System.Linq;
using System.Text;

using Microsoft.Kinect;
using myDrumStick;
using myDrumSet;
using myDrum;

namespace DrumPlayer
{

    /// <summary>
    /// Interaktivni logika aplikace pro Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {// mody aplikace a inicializace na mod uvodni obrazovky
        private enum AppMode { Start, Editor, Player };
        private AppMode Mode = AppMode.Start;

		// pomocne prommene pro gesto zmeny modu
        float slajd_x;
        float slajd_y;
        float maxR_x;
        float maxR_y;
        bool slajdRtoL;
		
        // uvodni zvuk pri spusteni aplikace
        private SoundPlayer welcomeSound = new SoundPlayer("../../Sounds/uvodni.wav");

		// pomocne promenne pro vykreslovani obrazovky
        private DrawingGroup drawingGroup;
        private DrawingImage imageSource;

        // hraci palicky pro pravou a levou ruku
        protected DrumStick drumStickLeft;
        protected DrumStick drumStickRight;

		// bici sestava
        protected DrumSet drumset;
		
        // souradnice prave a leve ruky
        private Point prava;
        private Point leva;

        // pomocne promenne pro zobrazovani dosazenych uspechu
        bool achievementRendering;
        int countAchivFrame;
        string achievementSrc;

        // kostra uzivatele
        bool closing = false;
        const int skeletonCount = 6;
        Skeleton[] allSkeletons = new Skeleton[skeletonCount];


        private WriteableBitmap outputBitmap;
        private static readonly int Bgr32BytesPerPixel = (PixelFormats.Bgr32.BitsPerPixel + 7) / 8;

        private const int RedIndex = 2;
        private const int GreenIndex = 1;
        private const int BlueIndex = 0;

		// rozhrani Kinectu
        KinectSensor kinectSensor;
        public Window1()
        {// inicializace souradnic prave a leve ruky
            prava = new Point(0, 0);
            leva = new Point(0, 0);

            // inicializace prommenych pro slajdovani modu aplikace
            slajd_x = 0.0f;
            maxR_x = 0.0f;
            slajd_y = 0.0f;
            maxR_y = 0.0f;
            slajdRtoL = false;

            // inicializace promennych pro achievmenty 
            achievementRendering = false;
            countAchivFrame = 0;
            achievementSrc = "";

            InitializeComponent();

            // inicializace rohrani Kinectu
            kinectSensor = KinectSensor.KinectSensors[0];
            kinectSensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);
            kinectSensor.Start();
            
            // udalost pro vykreslovani obrazovky
            kinectSensor.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(drawWindow);

            // udalost pro kazdy novy snimek ze senzoru
            kinectSensor.SkeletonStream.Enable();
            kinectSensor.AllFramesReady += new EventHandler<AllFramesReadyEventArgs>(processNewFrame);

            // inicializace bici soustavy
            setDrumSet();

            // spusteni uvodni hudby
            welcomeSound.Play();
            welcomeSound.PlayLooping();
        }

        // defaultni nastaveni bici soustavy
        void setDrumSet() 
        {
            this.drumset = new DrumSet();
            
            Drum newD;
            ImageSource imageSourceTmp;
            int i = 0;

            String baseSoundDir = "../../Sounds/zvuky/";
            String baseImageDir = "../../Images/source/";

            newD = new Drum(i += 50, 400, 50, 230);
            newD.setSoundPath(baseSoundDir + "gong.wav");
            imageSourceTmp = new BitmapImage(new Uri(baseImageDir + "gong.png", UriKind.Relative));
            newD.setImageSource(imageSourceTmp);
            drumset.addDrum(newD);            

            newD = new Drum(i += 50, 150, 100, 80);
            imageSourceTmp = new BitmapImage(new Uri(baseImageDir + "bubenMaly.png", UriKind.Relative));
            newD.setImageSource(imageSourceTmp);
            newD.setSoundPath(baseSoundDir +  "malyBuben.wav");
            drumset.addDrum(newD);

            newD = new Drum(i += 100, 150, 100, 110);
            newD.setSoundPath(baseSoundDir + "stredniBuben.wav");
            imageSourceTmp = new BitmapImage(new Uri(baseImageDir + "bubenMaly2.png", UriKind.Relative));
            newD.setImageSource(imageSourceTmp);
            drumset.addDrum(newD);

            newD = new Drum(i += 100, 150, 120, 100);
            imageSourceTmp = new BitmapImage(new Uri(baseImageDir + "bubenStredni.png", UriKind.Relative));
            newD.setImageSource(imageSourceTmp);
            newD.setSoundPath(baseSoundDir + "malyBuben2.wav");
            drumset.addDrum(newD);

            newD = new Drum(i +=120, 200, 100, 150);
            newD.setSoundPath(baseSoundDir + "velkyBuben.wav");
            imageSourceTmp = new BitmapImage(new Uri(baseImageDir + "bubenVelky-fliped.png", UriKind.Relative));
            newD.setImageSource(imageSourceTmp);
            drumset.addDrum(newD);

            newD = new Drum(i += 80, 330, 100, 350);
            newD.setSoundPath(baseSoundDir +  "cinely.wav");
            imageSourceTmp = new BitmapImage(new Uri(baseImageDir + "cinel2.png", UriKind.Relative));
            newD.setImageSource(imageSourceTmp);
            drumset.addDrum(newD);

            this.drumStickLeft = new DrumStick(kinectSensor, drumset);
            this.drumStickRight = new DrumStick(kinectSensor, drumset);
        }

        // Metoda vrati prvni nalezenou kostru uzivatele
        Skeleton GetFirstSkeleton(AllFramesReadyEventArgs e)
        {
            using (SkeletonFrame skeletonFrameData = e.OpenSkeletonFrame())
            {
                if (skeletonFrameData == null) return null;
                skeletonFrameData.CopySkeletonDataTo(allSkeletons);

                Skeleton first = (from s in allSkeletons
                                  where s.TrackingState == SkeletonTrackingState.Tracked
                                  select s).FirstOrDefault();
                return first;
            }
        }

        // udalost na novy snimek senzoru
        // kontrola gest
        void processNewFrame(object sender, AllFramesReadyEventArgs e)
        {
            if (closing) return;

            // ziskej kostru uzivatele
            Skeleton first = GetFirstSkeleton(e);
            if (first == null) return;

            // zpracovani gest pro ruzne mody aplikace
            
            if (Mode == AppMode.Start)
            {// kontrola presunu na dalsi mod
                checkGestSlide(first);
            }
            else if (Mode == AppMode.Editor)
            {// kontrola gesta pro presun bubnu
                this.checkGestDragAndDrop(first.Joints[JointType.HandLeft], first.Joints[JointType.HandRight], first.Joints[JointType.Head]);
                
                // kontrola presunu na dalsi mod
                checkGestSlide(first);
            }
            else if (Mode == AppMode.Player)
            {// hrani na bubny pro pravou a levou ruku
                this.drumStickLeft.checkGest(first.Joints[JointType.HandLeft]);
                this.drumStickRight.checkGest(first.Joints[JointType.HandRight]);
                // kontrola presunu na dalsi mod
                checkGestSlide(first);
            }
            
            // kontrola uspechu uzivatele
            checkAchievement();
  
        }

        // kontrola a zpracovani gesta pro presun bubnu
		
		protected Drum drumDrag = null;
        protected void checkGestDragAndDrop(Joint leftHand, Joint rightHand, Joint headPos)
        {// pozice hlavy a prave a leve ruky
            DepthImagePoint left  = this.kinectSensor.MapSkeletonPointToDepth(leftHand.Position,  DepthImageFormat.Resolution640x480Fps30);
            DepthImagePoint right = this.kinectSensor.MapSkeletonPointToDepth(rightHand.Position, DepthImageFormat.Resolution640x480Fps30);
            DepthImagePoint head  = this.kinectSensor.MapSkeletonPointToDepth(headPos.Position, DepthImageFormat.Resolution640x480Fps30);

            // prava nebo leva ruka je nad hlavou, zacatek gesta
            if (left.Y < head.Y || right.Y < head.Y)
            {
                // prehazovani nastroju
                if (right.Y < head.Y)
                {
                    DepthImagePoint tmp = left;
                    left = right;
                    right = tmp;
                }

                right.Y = 480 - right.Y;
                //Debug.WriteLine("hand ["+ right.X + ", "+ (right.Y) + "]");

                // pokud neni uchopeny zadny buben
                // zjisti, zda nemuzes nejaky uchopit
                if (this.drumDrag == null)
                {
                    foreach (Drum d in this.drumset.getDrums())
                    {
                        if (d.collision(right.X, right.Y))
                        {
                            this.drumDrag = d;
                            //Debug.WriteLine("hand [" + right.X + ", " + (right.Y) + "]");
                            break;
                        }
                    }
                }

                // pokud je nejaky buben uchopeny, zmen jeho pozici
                if (this.drumDrag != null)
                {
                    this.drumDrag.x = right.X - this.drumDrag.width / 2;
                    this.drumDrag.y = right.Y + this.drumDrag.heigth / 2;
                }
                
            }
            // pusti buben, konec gesta
            else
                this.drumDrag = null;

        }

        // kontrola gesta pro zmenu modu aplikace
        private void checkGestSlide(Skeleton first)
        {
            // prava ruka je v horni casti obrazovky
            if (slajd_x >= first.Joints[JointType.HandRight].Position.X)
            {
                // gesto nebylo zahajeno, zacatek gesta
                if (!slajdRtoL)
                {
                    slajdRtoL = true;
                    maxR_x = slajd_x;
                    maxR_y = slajd_y;

                }
                slajd_x = first.Joints[JointType.HandRight].Position.X;
                slajd_y = first.Joints[JointType.HandRight].Position.Y;
            }
            // konec gesta
            else
            {// pokud probehlo gesto pro zmenu modu, zmen mod aplikace
                if (slajdRtoL)
                {
                    float rozdil_x = maxR_x - first.Joints[JointType.HandRight].Position.X;
                    float rozdil_y = maxR_y - first.Joints[JointType.HandRight].Position.Y;
                    if (rozdil_x > 0.65 && rozdil_y < 0.13 && convertSkeletonPointToScreen(first.Joints[JointType.HandRight].Position).Y < 130)
                    {
                        Mode = ((int)(++Mode) >= 3) ? 0 : Mode;

                        // prehravani hudby na pozadi
                        if (Mode == AppMode.Player) this.welcomeSound.Stop();
                        else this.welcomeSound.PlayLooping();

                        if (Mode == AppMode.Editor) {
                            drumDrag = new Drum();
                        }
                    }
                }
                
                // nastaveni defaultnich hodnot
                slajdRtoL = false;
                maxR_x = 0.0f;
                maxR_y = 0.0f;
                slajd_x = first.Joints[JointType.HandRight].Position.X;
                slajd_y = first.Joints[JointType.HandRight].Position.Y;
            }
        }

        // prepocita bod kostry uzivatele na bod obrazovky
        private Point convertSkeletonPointToScreen(SkeletonPoint skelpoint)
        {
            DepthImagePoint depthPoint = this.kinectSensor.MapSkeletonPointToDepth(
                                            skelpoint,
                                            DepthImageFormat.Resolution640x480Fps30);
            return new Point(depthPoint.X, depthPoint.Y);
        }

        // vykreslovani snimku aplikace
        private void drawWindow(object sender, ColorImageFrameReadyEventArgs e)
        {
            // otevre snimek
            using (ColorImageFrame colorVideoFrame = e.OpenColorImageFrame())
            {
                if (colorVideoFrame == null) return;

                Byte[] pixelData = new Byte[colorVideoFrame.PixelDataLength];
                colorVideoFrame.CopyPixelDataTo(pixelData);

                this.outputBitmap = new WriteableBitmap(
                   colorVideoFrame.Width,
                   colorVideoFrame.Height,
                   96,  // DpiX
                   96,  // DpiY
                   PixelFormats.Bgr32,
                   null);

                this.outputBitmap.WritePixels(
                    new Int32Rect(0, 0, colorVideoFrame.Width, colorVideoFrame.Height),
                    pixelData,
                    colorVideoFrame.Width * Bgr32BytesPerPixel,
                    0);

                // vykreslovani objektu
                this.drawingGroup = new DrawingGroup();
                this.imageSource = new DrawingImage(this.drawingGroup);
                kinectDepthImage.Source = imageSource;

                using (DrawingContext dc = this.drawingGroup.Open())
                {
                    // pozadi 
                    dc.DrawImage(outputBitmap, new Rect(0.0, 0.0, 640, 480));

                    // mody s bubnama
                    if (Mode == AppMode.Editor || Mode == AppMode.Player)
                    {
                        // vykresli vsechny  bubny
                        foreach (Drum drum in this.drumset.getDrums())
                        {
                            drum.drawDrum(dc);
                        }

                        // vykresli dosazene uspechy
                        if (achievementRendering)
                        {
                            countAchivFrame++;
                            ImageSource achievement1 = new BitmapImage(new Uri(achievementSrc, UriKind.Relative));
                            dc.DrawImage(achievement1, new Rect(320 - achievement1.Width / 4, 20, achievement1.Width / 2, achievement1.Height / 2));

                            if (countAchivFrame >= 100)
                            {
                                countAchivFrame = 0;
                                achievementRendering = false;
                            }
                        }
                    }
                    
                    // vykresli nazev modu
                    
                    if (Mode == AppMode.Start) {
                        ImageSource imageTitleSource = new BitmapImage(new Uri(@"../../Images/source/title.png", UriKind.Relative));
                        dc.DrawImage(imageTitleSource, new Rect(0, 0, 640, 480));
                    }
                    else if (Mode == AppMode.Editor)
                    {
                        ImageSource imageTitleSource = new BitmapImage(new Uri(@"../../Images/source/editor.png", UriKind.Relative));
                        dc.DrawImage(imageTitleSource, new Rect(0, 0, 640, 480));
                    }
                    else
                    {
                        ImageSource imageTitleSource = new BitmapImage(new Uri(@"../../Images/source/player.png", UriKind.Relative));
                        dc.DrawImage(imageTitleSource, new Rect(0, 0, 640, 480));
                    }

                    // vykresli hranici
                    this.drawingGroup.ClipGeometry = new RectangleGeometry(new Rect(0.0, 0.0, 640.0, 480.0));
                }
            }
        }

        // kontrola dosazenych uspechu
        private void checkAchievement()
        {
            string basedir = "../../Images/Achievments/";

            if ((this.drumStickLeft.getHits() + this.drumStickRight.getHits()) == 10)
            {
                achievementSrc = basedir + "great_start.png";
                achievementRendering = true;
            }

            if (this.drumStickLeft.getHits() == 15 && this.drumStickRight.getHits() == 0 ||
               this.drumStickRight.getHits() == 15 && this.drumStickLeft.getHits() == 0)
            {
                achievementSrc = basedir + "one_hand.png";
                achievementRendering = true;
            }

            if (this.drumStickLeft.getHits() == 50 && this.drumStickRight.getHits() == 0 ||
               this.drumStickRight.getHits() == 50 && this.drumStickLeft.getHits() == 0)
            {
                achievementSrc = basedir + "oneHandBand.png";
                achievementRendering = true;
            }

            if ((this.drumStickLeft.getHits() + this.drumStickRight.getHits()) == 50)
            {
                achievementSrc = basedir + "50hits.png";
                achievementRendering = true;
            }

            if ((this.drumStickLeft.getHits() + this.drumStickRight.getHits()) == 100)
            {
                achievementSrc = basedir + "100hits.png";
                achievementRendering = true;
            }

            // dalsi akce
            if (achievementRendering)
            {
                
            }
        }

        // vyjimky
        
        private void image1_ImageFailed(object sender, ExceptionRoutedEventArgs e)
        {
            return;
        }

        private void drum_ImageFailed(object sender, ExceptionRoutedEventArgs e)
        {
            return;
        }

        private void image1_ImageFailed_1(object sender, ExceptionRoutedEventArgs e)
        {
            return;
        }
    }
}