﻿///----------------------------------------------------------
///Kinect Depth Example
///Date: February 12th, 2012
///--------------------
///Authors:
/// - Jesus Dominguez:
/// 
/// - Angel Hernandez:
///     @mifulapirus
///     www.tupperbot.com
///-----------------------
///All code is based on the Kinect Explorer Example. 
///-----------------------------------------------------------------------------------------------
///Summary:
///This is just a simplification of the Explorer Example code that comes
///with the final version of the Kinect SDK released by Microsoft in February.
///This example shows the minimum code required to get the depth image drawn in a WPF program.
///It is for sure not a perfect and super safe code, but it is just intended to show how to do 
///this in a simple way.
///-----------------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Microsoft.Kinect;

using System.Diagnostics;

using System.Linq;
using System.Text;
using System.Windows.Documents;
using System.Windows.Navigation;
using System.Windows.Shapes;

//using System.Diagnostics;

using myDrumStick;
using myDrumSet;
using myDrum;

namespace WPFKinectTest
{

    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private DrawingGroup drawingGroup;
        private DrawingImage imageSource;

        protected DrumStick drumStickLeft;
        protected DrumStick drumStickRight;

        protected DrumSet drumset;

        private Point prava;
        private Point leva;

        bool closing = false;
        const int skeletonCount = 6;
        Skeleton[] allSkeletons = new Skeleton[skeletonCount];

        //The bitmap that will contain the actual converted depth into an image
        private WriteableBitmap outputBitmap;
        private static readonly int Bgr32BytesPerPixel = (PixelFormats.Bgr32.BitsPerPixel + 7) / 8;

        //Format of the last Depth Image. 
        //This changes when you first run the program or whenever you minimize the window 
        private DepthImageFormat lastImageFormat;

        //Identify each color layer on the R G B
        private const int RedIndex = 2;
        private const int GreenIndex = 1;
        private const int BlueIndex = 0;

        //Declare our Kinect Sensor!
        KinectSensor kinectSensor;

        public Window1()
        {
            prava = new Point(0, 0);
            leva = new Point(0, 0);

            InitializeComponent();

            //Select the first kinect found
            kinectSensor = KinectSensor.KinectSensors[0];
            //Set up the depth stream to be the largest possible
            kinectSensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);
            //Initialize the Kinect Sensor
            kinectSensor.Start();
            //Subscribe to an event that will be triggered every time a new frame is ready
            kinectSensor.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(ColorImageReady);

            //event pro skeleton
            kinectSensor.SkeletonStream.Enable();
            kinectSensor.AllFramesReady += new EventHandler<AllFramesReadyEventArgs>(sensor_AllFramesReady);
            
            drumset = new DrumSet();

            drumset.addDrum(new Drum(250, 150, 200, 150));

            Drum newD = new Drum(0, 130, 200, 150);
            newD.setSoundPath("../../Sounds/Kickz/Kick (9).wav");
            drumset.addDrum(newD);

            newD = new Drum(440, 130, 200, 150);
            newD.setSoundPath("../../Sounds/Kickz/Kick (12).wav");
            drumset.addDrum(newD);

            this.drumStickLeft = new DrumStick(kinectSensor, drumset);
            this.drumStickRight = new DrumStick(kinectSensor, drumset);
        }


        Skeleton GetFirstSkeleton(AllFramesReadyEventArgs e)
        {
            using (SkeletonFrame skeletonFrameData = e.OpenSkeletonFrame())
            {
                if (skeletonFrameData == null)
                {
                    return null;
                }


                skeletonFrameData.CopySkeletonDataTo(allSkeletons);

                //get the first tracked skeleton

                Skeleton first = (from s in allSkeletons
                                  where s.TrackingState == SkeletonTrackingState.Tracked
                                  select s).FirstOrDefault();

                return first;

            }
        }


        void sensor_AllFramesReady(object sender, AllFramesReadyEventArgs e)
        {
            if (closing)
            {
                return;
            }

            //Get a skeleton
            Skeleton first = GetFirstSkeleton(e);

            if (first == null)
            {
                return;
            }

            this.drumStickLeft.checkGest(first.Joints[JointType.HandLeft]);
            this.drumStickRight.checkGest(first.Joints[JointType.HandRight]);

            //ulozeni pozic rukou pro vykresleni jejich vykresleni
            this.leva = SkeletonPointToScreen(first.Joints[JointType.HandLeft].Position);
            this.prava = SkeletonPointToScreen(first.Joints[JointType.HandRight].Position);

        }

        private Point SkeletonPointToScreen(SkeletonPoint skelpoint)
        {
            // Convert point to depth space.  
            // We are not using depth directly, but we do want the points in our 640x480 output resolution.
            DepthImagePoint depthPoint = this.kinectSensor.MapSkeletonPointToDepth(
                                                                             skelpoint,
                                                                             DepthImageFormat.Resolution640x480Fps30);
            return new Point(depthPoint.X, depthPoint.Y);
        }

        private void ColorImageReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            //open image
            using (ColorImageFrame colorVideoFrame = e.OpenColorImageFrame())
            {

                if (colorVideoFrame == null)
                    return;

                //Create array for pixel data and copy it from the image frame
                Byte[] pixelData = new Byte[colorVideoFrame.PixelDataLength];
                colorVideoFrame.CopyPixelDataTo(pixelData);

                //Create the new Bitmap
                this.outputBitmap = new WriteableBitmap(
                   colorVideoFrame.Width,
                   colorVideoFrame.Height,
                   96,  // DpiX
                   96,  // DpiY
                   PixelFormats.Bgr32,
                   null);

                //this.kinectDepthImage.Source = this.outputBitmap;

                //Copy the RGB matrix to the bitmap to make it visible
                this.outputBitmap.WritePixels(
                    new Int32Rect(0, 0, colorVideoFrame.Width, colorVideoFrame.Height),
                    pixelData,
                    colorVideoFrame.Width * Bgr32BytesPerPixel,
                    0);

                //Vykreslovani objektu
                this.drawingGroup = new DrawingGroup();
                this.imageSource = new DrawingImage(this.drawingGroup);
                kinectDepthImage.Source = imageSource;

                using (DrawingContext dc = this.drawingGroup.Open())
                {
                    //pozadí
                    dc.DrawImage(outputBitmap, new Rect(0.0, 0.0, 640, 480));

                    //vykresleni leva ruka
                    dc.DrawEllipse(Brushes.Green,
                                null,
                               leva,
                                10,
                                10);

                    //vykresleni prava ruka
                    dc.DrawEllipse(Brushes.Blue,
                                null,
                               prava,
                                10,
                                10);


                    //Vykresleni jeden buben
                    ImageSource imageSourceTmp = new BitmapImage(new Uri(@"../../Images/drum.png", UriKind.Relative));
                    foreach (Drum drum in this.drumset.getDrums()) {
                        dc.DrawImage(imageSourceTmp, new Rect(drum.x, 480 - drum.y, drum.width, drum.heigth));
                    }

                    //Vykresleni hranice
                    this.drawingGroup.ClipGeometry = new RectangleGeometry(new Rect(0.0, 0.0, 640.0, 480.0));
                }
            }
        }



        private void image1_ImageFailed(object sender, ExceptionRoutedEventArgs e)
        {

        }

        private void drum_ImageFailed(object sender, ExceptionRoutedEventArgs e)
        {

        }

        private void image1_ImageFailed_1(object sender, ExceptionRoutedEventArgs e)
        {

        }
    }
}